import { useState, useMemo, useRef, useEffect } from 'react';
import { useFrame } from '@react-three/fiber';
import { useFBX, useAnimations } from '@react-three/drei';
import * as THREE from 'three';
import Box from './box';
import * as Common from '../utils/common';
import * as TWEEN from "@tweenjs/tween.js";
import { INTERVAL_TIME } from '@/config/setting'

const Agv = (props) => {
    const { position, theta, height, loaderTheta, forkLength, trayList, hasBox } = props;

    const [groupPos, setGroupPos] = useState({ x: 0, y: 0, z: 0 });
    const [groupTheta, setGroupTheta] = useState(0);
    const [loaderHeight, setLoaderHeight] = useState(0);

    useEffect(() => {
        if (!props || Object.keys(props).length === 0) {
            return;
        }
        // group position
        const pos = {
            x: position?.[0],
            y: position?.[1],
            z: position?.[2],
        }
        if (!Common.deepEqual(groupPos, pos)) {
            new TWEEN.Tween(groupPos)
                .to({
                    x: position[0],
                    y: position[1],
                    z: position[2]
                }, INTERVAL_TIME)
                .easing(TWEEN.Easing.Linear.None)
                .onUpdate((e) => {
                    setGroupPos({ ...e });
                })
                .onComplete((e) => {
                    setGroupPos({ ...e });
                }).start();
        }

        // group theta
        if (groupTheta !== theta) {
            const minTheta = Common.minDiffTheta(groupTheta, theta);
            new TWEEN.Tween({ value: groupTheta })
                .to({ value: minTheta }, INTERVAL_TIME)
                .easing(TWEEN.Easing.Linear.None)
                .onUpdate((e) => {
                    setGroupTheta(e.value)
                })
                .onComplete((e) => {
                    setGroupTheta(e.value)
                }).start();
        }

        // loader height
        if (loaderHeight !== height) {
            new TWEEN.Tween({ value: loaderHeight })
                .to({ value: height }, INTERVAL_TIME)
                .easing(TWEEN.Easing.Linear.None)
                .onUpdate((e) => {
                    setLoaderHeight(e.value)
                })
                .onComplete((e) => {
                    setLoaderHeight(e.value)
                }).start();
        }

    }, [props]);

    useFrame((state, delta) => {
        TWEEN.update();
    })

    const bodyModel = useMemo(() => {
        const fbx = useFBX('/models/agv/body.fbx');
        if (!fbx.castShadow) {
            Common.setShadow(fbx);
        }
        return fbx.clone();
    }, [])

    const loaderModel = useMemo(() => {
        const fbx = useFBX('/models/agv/loader.fbx');
        if (!fbx.castShadow) {
            Common.setShadow(fbx);
        }
        return fbx.clone();
    }, [])

    const forkModel = useMemo(() => {
        const fbx = useFBX('/models/agv/fork.fbx');
        if (!fbx.castShadow) {
            Common.setShadow(fbx);
        }
        return fbx.clone();
    }, [])

    return (
        <>
            <group
                rotation-y={Common.rotationParseNum(groupTheta)}
                scale={0.5}
                position={[groupPos.x, groupPos.y, groupPos.z]}
            >
                <primitive object={bodyModel} castShadow />
                <group
                    position-y={loaderHeight}
                >
                    <primitive object={loaderModel} castShadow position={[0, 50, 0]} />
                    <primitive object={forkModel} castShadow position={[0, 61, 0]} />
                    <Box position={[100, 100, 300]} />
                </group>
            </group>
        </>
    )
}

export default Agv;